package com.netiq.nam.oauth.helpers;

import com.google.api.client.auth.oauth2.*;
import com.google.api.client.http.GenericUrl;
import com.google.api.client.http.HttpResponse;
import com.google.api.client.util.store.DataStore;
import com.google.api.client.util.store.MemoryDataStoreFactory;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

/**
 * Created by tsureshkumar on 12/7/14.
 */
public class OAuth2Helper {
    private static final Logger logger = Logger.getLogger(OAuth2Helper.class.getName());
    public static DataStore<StoredCredential> DATASTORE;

    static {
        try {
            DATASTORE = MemoryDataStoreFactory.getDefaultInstance().getDataStore("mem");
            //DATASTORE = StoredCredential.getDefaultDataStore(new FileDataStoreFactory(new File("datastoredir")));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static Collection<String> getScopes() {
        return scopes;
    }

    private static Collection<String> scopes = Arrays.asList("profile", "email");

    private final AuthorizationCodeFlow flow;

    public static String getAccessToken(String user) throws IOException {
        StoredCredential storedCredential = DATASTORE.get(user);
        return (storedCredential != null) ? storedCredential.getAccessToken() : null;
    }

    public static void removeSession(String id) throws IOException {
        DATASTORE.delete(id);
    }

    private static AuthorizationCodeFlow getAuthorizationCodeFlow() {
        return new AuthorizationCodeFlow.Builder(BearerToken.authorizationHeaderAccessMethod(),
                Http.INSECURE_TRANSPORT,
                Http.JSON_FACTORY,
                new GenericUrl(OAuth2Properties.tokenEndPoint()),
                new ClientParametersAuthentication(OAuth2Properties.clientID(), OAuth2Properties.clientSecret()),
                OAuth2Properties.clientID(),
                OAuth2Properties.authorizationEndpoint())
                .setCredentialDataStore(OAuth2Helper.DATASTORE)
                .setScopes(scopes)
                .build();
    }

    private static OAuth2Helper instance = null;

    public OAuth2Helper() {
        flow = getAuthorizationCodeFlow();
    }


    public String buildRedirectionURL(String redirectURL, String sessionID, String state) {
        return flow.newAuthorizationUrl()
                .setState(state)
                .setRedirectUri(redirectURL)
                .build();
    }

    public String exchangeCodeForToken(String code, String redirectURL, String sessionID) {
        logger.info(redirectURL);
        logger.info(code);

        AuthorizationCodeTokenRequest tokenreq = flow.newTokenRequest(code).setRedirectUri(redirectURL);
        TokenResponse response = null;
        try {
            response = tokenreq.execute();
            flow.createAndStoreCredential(response, sessionID);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
        return response.getAccessToken();
    }

    public Map getUserInfo(String sessionID) throws IOException {
        String accessToken = DATASTORE.get(sessionID).getAccessToken();
        String url = OAuth2Properties.userInfoEndpoint();
        HttpResponse api = new Http().insecureGet(url, accessToken);
        return api.parseAs(HashMap.class);
    }

    public synchronized static OAuth2Helper getInstance() {
        if (instance == null) {
            instance = new OAuth2Helper();
        }
        return instance;
    }

    public StoredCredential getCredential(String id) throws IOException {
        return flow.getCredentialDataStore().get(id);
    }

    public AuthorizationCodeFlow getFlow() {
        return flow;
    }
}
