package com.netiq.nam.oauth.servlets;

import com.google.api.client.auth.oauth2.*;
import com.google.api.client.extensions.servlet.auth.oauth2.AbstractAuthorizationCodeCallbackServlet;
import com.google.api.client.http.GenericUrl;
import com.netiq.nam.oauth.helpers.Http;
import com.netiq.nam.oauth.helpers.OAuth2Helper;
import com.netiq.nam.oauth.helpers.OAuth2Properties;
import com.netiq.nam.oauth.helpers.Token;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * Created by tsureshkumar on 12/7/14.
 */
public class AuthorizationCallback extends AbstractAuthorizationCodeCallbackServlet {

    public static final String USER_ID = "USER_ID";
    public static final String OAUTH_2_TOKEN = "OAUTH2_TOKEN";

    @Override
    protected void onSuccess(HttpServletRequest req, HttpServletResponse resp, Credential credential)
            throws ServletException, IOException {
        Token token = new Http().insecureGet(OAuth2Properties.tokenInfoEndpoint(), credential.getAccessToken()).parseAs(Token.class);
        if (token != null) {
            req.getSession(true).setAttribute(USER_ID, token.getUser());
            req.getSession(true).setAttribute(OAUTH_2_TOKEN, token);
        } else {
            resp.sendError(500);
            resp.getWriter().write("error getting token info");
        }
        resp.sendRedirect("/");
    }

    @Override
    protected void onError(
            HttpServletRequest req, HttpServletResponse resp, AuthorizationCodeResponseUrl errorResponse)
            throws ServletException, IOException {
        resp.sendError(500);
        resp.getWriter().write("error getting token");
    }

    @Override
    protected String getRedirectUri(HttpServletRequest req) throws ServletException, IOException {
        GenericUrl url = new GenericUrl(req.getRequestURL().toString());
        url.setRawPath("/oauth2callback");
        return url.build();
    }

    @Override
    protected AuthorizationCodeFlow initializeFlow() throws IOException {
        return OAuth2Helper.getInstance().getFlow();
    }



    @Override
    protected String getUserId(HttpServletRequest req) throws ServletException, IOException {
        return (String) req.getSession(true).getId();
    }
}