import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.api.client.auth.oauth2.ClientCredentialsTokenRequest;
import com.google.api.client.auth.oauth2.ClientParametersAuthentication;
import com.google.api.client.auth.oauth2.PasswordTokenRequest;
import com.google.api.client.auth.oauth2.TokenResponse;
import com.google.api.client.http.GenericUrl;
import com.google.api.client.http.HttpResponse;
import javafx.scene.layout.VBox;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.*;

/**
 * Created by tsureshkumar on 8/27/14.
 */
public class Main {


    public static void main(String[] args) {

        final JFrame win = new JFrame("Client Credentials Sample");
        win.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        win.setSize(new Dimension(300, 500));
        win.setLayout(new BorderLayout());

        Box vbox;
        win.add(vbox = Box.createVerticalBox(), BorderLayout.CENTER);

        Box controlBox;
        Box ubox;
        win.add(controlBox = Box.createVerticalBox(), BorderLayout.PAGE_START);

        controlBox.add(ubox = Box.createHorizontalBox());
        final JTextField txtUsername;
        final JPasswordField txtPassword;
        ubox.add(new JLabel("Client ID:"));
        ubox.add(txtUsername = new JTextField());
        ubox.add(new JLabel("Client Secret:"));
        ubox.add(txtPassword = new JPasswordField());

        controlBox.add(ubox = Box.createHorizontalBox());
        ubox.add(new JLabel("Scopes (space seperated):"));
        final JTextField txtScopes;
        ubox.add(txtScopes = new JTextField("email profile"));


        controlBox.add(ubox = Box.createHorizontalBox());
        JButton btn;
        ubox.add(btn = new JButton("Get Token"));
        JButton btnTokenDetails;
        ubox.add(btnTokenDetails = new JButton("Get Token Details"));

        controlBox.add(ubox = Box.createHorizontalBox());
        ubox.add(new JLabel("Token:"));
        final JTextField txtToken;
        ubox.add(txtToken = new JTextField(""));

        JPanel frame;
        vbox.add(frame = new JPanel());
        frame.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        frame.setSize(new Dimension(300, 200));
        final JLabel lbl;
        frame.add(lbl = new JLabel());

        win.show();


        final Properties properties = NAMOAuthHelper.loadProperties();
        final TokenResponse[] response = {null};

        txtUsername.setText(properties.getProperty("client_id"));
        txtPassword.setText(properties.getProperty("client_secret"));
        btn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                ClientCredentialsTokenRequest ccReq = new ClientCredentialsTokenRequest(
                        NAMOAuthHelper.TRANSPORT,
                        NAMOAuthHelper.JSON_FACTORY,
                        new GenericUrl(properties.getProperty("token_endpoint"))
                )
                        .setScopes(Arrays.asList(txtScopes.getText().split(" ")))
                        .setClientAuthentication(new ClientParametersAuthentication(
                        		txtUsername.getText(),
                        		String.copyValueOf(txtPassword.getPassword())));
                try {
                    response[0] = ccReq.execute();
                    String value = response[0].getAccessToken();
                    System.out.println(value);
                    txtToken.setText(value);

                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        });

        btnTokenDetails.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(response[0] == null && txtToken.getText() == null) {
                    JOptionPane.showMessageDialog(win, "No Token Info available");
                    return;
                }
                TokenResponse resp = response[0];
                HttpResponse userResponse = null;
                try {
                    userResponse = NAMOAuthHelper.executeGet(NAMOAuthHelper.TRANSPORT,
                            NAMOAuthHelper.JSON_FACTORY,
                           txtToken.getText(),
                            new GenericUrl(properties.getProperty("tokeninfo_endpoint")));
                    if (userResponse.isSuccessStatusCode()) {
                        String value = userResponse.parseAsString();
                        ObjectMapper mapper = new ObjectMapper();
                        value = mapper.writerWithDefaultPrettyPrinter().writeValueAsString(mapper.readValue(value, Object.class));
                        String html = "<html><pre>" + value + "</pre></html>";
                        System.out.println(html);
                        lbl.setText(html);
                    } else {
                        JOptionPane.showMessageDialog(win, userResponse.getStatusMessage());
                    }
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        });

    }
}
